#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from collections.abc import Iterable
from datetime import date

from holidays.calendars.sinhala import _SinhalaLunar
from holidays.groups.eastern import EasternCalendarHolidays


class SinhalaCalendarHolidays(EasternCalendarHolidays):
    """
    Sinhala holidays.

    Sinhala Buddhist Uposatha day calculation method is different from Thai LuniSolar
    and Buddhist (Mahayana) used in East Asia.

    Due to the fact that Poya (Uposatha) days are calculated astronomically
    based on how close a particular day is closest to full moon at noon, and that
    an extra month is added every 33 months interval, this is hardcoded for now.

    Adhi month dates are instead hardcoded in Sri Lanka country implementation.
    """

    def __init__(self, cls=None, *, show_estimated=False) -> None:
        self._sinhala_calendar = cls() if cls else _SinhalaLunar()
        self._sinhala_calendar_show_estimated = show_estimated

    def _add_sinhala_calendar_holiday(
        self, name: str, dt_estimated: tuple[date | None, bool]
    ) -> date | None:
        """
        Add Sinhala calendar holiday.

        Adds customizable estimation label to holiday name if holiday date
        is an estimation.
        """
        return self._add_eastern_calendar_holiday(
            name, dt_estimated, show_estimated=self._sinhala_calendar_show_estimated
        )

    def _add_sinhala_calendar_holiday_set(
        self, name: str, dts_estimated: Iterable[tuple[date, bool]], days_delta: int = 0
    ) -> set[date]:
        """
        Add Sinhala calendar holidays.

        Adds customizable estimation label to holiday name if holiday date
        is an estimation.
        """
        return self._add_eastern_calendar_holiday_set(
            name,
            dts_estimated,
            show_estimated=self._sinhala_calendar_show_estimated,
            days_delta=days_delta,
        )

    def _add_bak_poya(self, name) -> date | None:
        """
        Add Bak Poya (first full moon day of the 5th lunar month).

        https://web.archive.org/web/20250417183100/https://us.lakpura.com/pages/bak-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.bak_poya_date(self._year)
        )

    def _add_binara_poya(self, name) -> date | None:
        """
        Add Binara Poya (first full moon day of the 10th lunar month).

        https://web.archive.org/web/20250415144613/https://us.lakpura.com/pages/binara-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.binara_poya_date(self._year)
        )

    def _add_duruthu_poya(self, name) -> set[date]:
        """
        Add Duruthu Poya (first full moon day of the 2nd lunar month).

        https://web.archive.org/web/20250417123343/https://us.lakpura.com/pages/duruthu-poya
        """
        return self._add_sinhala_calendar_holiday_set(
            name, self._sinhala_calendar.duruthu_poya_date(self._year)
        )

    def _add_esala_poya(self, name) -> date | None:
        """
        Add Esala Poya (first full moon day of the 8th lunar month).

        https://web.archive.org/web/20250415231927/https://us.lakpura.com/pages/esala-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.esala_poya_date(self._year)
        )

    def _add_il_poya(self, name) -> date | None:
        """
        Add Il Poya (first full moon day of the 12th lunar month).

        Also known as "Ill Poya"
        https://web.archive.org/web/20250415054940/https://us.lakpura.com/pages/il-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.il_poya_date(self._year)
        )

    def _add_medin_poya(self, name) -> date | None:
        """
        Add Medin Poya (first full moon day of the 4th lunar month).

        https://web.archive.org/web/20250415154545/https://us.lakpura.com/pages/medin-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.medin_poya_date(self._year)
        )

    def _add_nawam_poya(self, name) -> date | None:
        """
        Add Nawam Poya (first full moon day of the 3rd lunar month).

        Also known as "Navam Poya" and "Magha Puja".
        https://web.archive.org/web/20250416122014/https://us.lakpura.com/pages/navam-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.nawam_poya_date(self._year)
        )

    def _add_nikini_poya(self, name) -> date | None:
        """
        Add Nikini Poya (first full moon day of the 9th lunar month).

        https://web.archive.org/web/20241204065104/https://us.lakpura.com/pages/nikini-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.nikini_poya_date(self._year)
        )

    def _add_poson_poya(self, name) -> date | None:
        """
        Add Poson Poya (first full moon day of the 7th lunar month).

        https://web.archive.org/web/20250416105848/https://us.lakpura.com/pages/poson
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.poson_poya_date(self._year)
        )

    def _add_unduvap_poya(self, name) -> date | None:
        """
        Add Unduvap Poya (first full moon day of the 1st lunar month).

        Also known as "Undawap Poya".
        https://web.archive.org/web/20250413191847/https://us.lakpura.com/pages/unduvap-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.unduvap_poya_date(self._year)
        )

    def _add_vap_poya(self, name) -> date | None:
        """
        Add Vap Poya (first full moon day of the 11th lunar month).

        https://web.archive.org/web/20250418083429/https://us.lakpura.com/pages/vap-poya
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.vap_poya_date(self._year)
        )

    def _add_vesak_poya(self, name) -> date | None:
        """
        Add Vesak Poya (first full moon day of the 6th lunar month).

        Also known as "Wesak Poya".
        https://web.archive.org/web/20250419054525/https://us.lakpura.com/pages/vesak
        """
        return self._add_sinhala_calendar_holiday(
            name, self._sinhala_calendar.vesak_poya_date(self._year)
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.south_sudan import SouthSudan
from tests.common import CommonCountryTests


class TestSouthSudan(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SouthSudan)

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in self.full_range))

    def test_peace_agreement_day(self):
        self.assertHolidayName(
            "Peace Agreement Day", (f"{year}-01-09" for year in self.full_range)
        )

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Holy Saturday"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Easter Sunday"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_international_labor_day(self):
        self.assertHolidayName(
            "International Labour Day", (f"{year}-05-01" for year in self.full_range)
        )

    def test_spla_day(self):
        self.assertHolidayName("SPLA Day", (f"{year}-05-16" for year in self.full_range))

    def test_independence_day(self):
        self.assertHolidayName("Independence Day", (f"{year}-07-09" for year in self.full_range))

    def test_martyrs_day(self):
        self.assertHolidayName("Martyrs' Day", (f"{year}-07-30" for year in self.full_range))

    def test_christmas_eve(self):
        self.assertHolidayName("Christmas Eve", (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_second_day_of_christmas(self):
        self.assertHolidayName(
            "Second Day of Christmas", (f"{year}-12-26" for year in self.full_range)
        )

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-22",
            "2021-05-13",
            "2022-05-03",
            "2023-04-21",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-18",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr_holiday(self):
        name = "Eid al-Fitr Holiday"
        self.assertNoHolidayName(name)
        self.assertIslamicIslamicNoEstimatedHolidayName(
            name,
            "2020-05-23",
            "2021-05-14",
            "2022-05-04",
            "2023-04-22",
            "2024-04-11",
            "2025-04-01",
        )
        self.assertIslamicIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha_holiday(self):
        name = "Eid al-Adha Holiday"
        self.assertNoHolidayName(name)
        self.assertIslamicIslamicNoEstimatedHolidayName(
            name,
            "2020-08-01",
            "2021-07-19",
            "2022-07-10",
            "2023-06-29",
            "2024-06-17",
            "2025-06-07",
        )
        self.assertIslamicIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-01-09", "Peace Agreement Day"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-11", "Holy Saturday"),
            ("2020-04-12", "Easter Sunday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-05-01", "International Labour Day"),
            ("2020-05-16", "SPLA Day"),
            ("2020-05-22", "Eid al-Fitr"),
            ("2020-07-09", "Independence Day"),
            ("2020-07-30", "Martyrs' Day"),
            ("2020-07-31", "Eid al-Adha"),
            ("2020-12-24", "Christmas Eve"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Second Day of Christmas"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.groups import TibetanCalendarHolidays
from holidays.holiday_base import HolidayBase


class TestTibetanCalendarHolidays(TestCase):
    def test_add_tibetan_calendar_holiday(self):
        # Check for out-of-range dates.
        class TestHolidays(HolidayBase, TibetanCalendarHolidays):
            def __init__(self, *args, **kwargs):
                TibetanCalendarHolidays.__init__(self)
                super().__init__(*args, **kwargs)

        test_holidays = TestHolidays()

        test_holidays._populate(2100)
        test_holidays._add_blessed_rainy_day("Blessed Rainy Day")
        test_holidays._add_birth_of_guru_rinpoche("Birth of Guru Rinpoche")
        test_holidays._add_buddha_first_sermon("Buddha First Sermon")
        test_holidays._add_buddha_parinirvana("Buddha Parinirvana")
        test_holidays._add_day_of_offering("Day of Offering")
        test_holidays._add_death_of_zhabdrung("Death of Zhabdrung")
        test_holidays._add_descending_day_of_lord_buddha("Descending Day of Lord Buddha")
        test_holidays._add_losar("Losar")
        test_holidays._add_thimphu_drubchen_day("Thimphu Drubchoe")
        test_holidays._add_thimphu_tshechu_day("Thimphu Tshechu")
        test_holidays._add_tibetan_winter_solstice("Winter Solstice Day")
        self.assertEqual(0, len(test_holidays))



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.saint_vincent_and_the_grenadines import (
    SaintVincentAndTheGrenadines,
)
from tests.common import CommonCountryTests


class TestSaintVincentAndTheGrenadines(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1979, 2050)
        super().setUpClass(SaintVincentAndTheGrenadines, years=years, years_non_observed=years)

    def test_special_holidays(self):
        self.assertHolidayName("Public Health Holiday", "2021-01-22", "2021-01-25")

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1979, 2050)))
        obs_dt = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_national_heroes_day(self):
        name = "National Heroes' Day"
        self.assertHolidayName(name, (f"{year}-03-14" for year in range(1979, 2050)))
        obs_dt = (
            "2004-03-15",
            "2010-03-15",
            "2021-03-15",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1979, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1979, 2050))

    def test_national_workers_day(self):
        name = "National Workers' Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1979, 2050)))
        obs_dt = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_spiritual_baptist_day(self):
        name = "National Spiritual Baptist Day"
        self.assertHolidayName(name, (f"{year}-05-21" for year in range(2025, 2050)))
        self.assertNoHolidayName(name, range(1979, 2025))

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, range(1979, 2050))

    def test_carnival_monday(self):
        name = "Carnival Monday"
        self.assertHolidayName(
            name,
            "2020-08-03",
            "2021-09-06",
            "2022-07-04",
            "2023-07-10",
            "2024-07-08",
            "2025-07-07",
        )
        self.assertHolidayName(name, range(1979, 2050))

    def test_carnival_tuesday(self):
        name = "Carnival Tuesday"
        self.assertHolidayName(
            name,
            "2020-08-04",
            "2021-09-07",
            "2022-07-05",
            "2023-07-11",
            "2024-07-09",
            "2025-07-08",
        )
        self.assertHolidayName(name, range(1979, 2050))

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(name, (f"{year}-08-01" for year in range(1979, 2050)))
        obs_dt = (
            "2004-08-02",
            "2010-08-02",
            "2021-08-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-10-27" for year in range(1979, 2050)))
        obs_dt = (
            "2002-10-28",
            "2013-10-28",
            "2019-10-28",
            "2024-10-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(1979, 2050)))
        obs_dt = (
            "2005-12-27",
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(1979, 2050)))
        obs_dt = (
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_2024_holidays(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-03-14", "National Heroes' Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-01", "National Workers' Day"),
            ("2024-05-20", "Whit Monday"),
            ("2024-07-08", "Carnival Monday"),
            ("2024-07-09", "Carnival Tuesday"),
            ("2024-08-01", "Emancipation Day"),
            ("2024-10-27", "Independence Day"),
            ("2024-10-28", "Independence Day (observed)"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-03-14", "National Heroes' Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "National Workers' Day"),
            ("2025-05-21", "National Spiritual Baptist Day"),
            ("2025-06-09", "Whit Monday"),
            ("2025-07-07", "Carnival Monday"),
            ("2025-07-08", "Carnival Tuesday"),
            ("2025-08-01", "Emancipation Day"),
            ("2025-10-27", "Independence Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-03-14", "National Heroes' Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "National Workers' Day"),
            ("2025-05-21", "National Spiritual Baptist Day"),
            ("2025-06-09", "Whit Monday"),
            ("2025-07-07", "Carnival Monday"),
            ("2025-07-08", "Carnival Tuesday"),
            ("2025-08-01", "Emancipation Day"),
            ("2025-10-27", "Independence Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cambodia import Cambodia
from tests.common import CommonCountryTests


class TestCambodia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Cambodia)

    def test_special_holidays(self):
        self.assertHoliday(
            "2016-05-02",
            "2016-05-16",
            "2018-05-21",
            "2019-09-30",
            "2020-05-11",
            "2020-08-17",
            "2020-08-18",
            "2020-08-19",
            "2020-08-20",
            "2020-08-21",
            "2024-04-16",
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "ទិវាចូលឆ្នាំសាកល"),
            ("2022-01-07", "ទិវាជ័យជម្នះលើរបបប្រល័យពូជសាសន៍"),
            ("2022-03-08", "ទិវាអន្តរជាតិនារី"),
            ("2022-04-14", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2022-04-15", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2022-04-16", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2022-05-01", "ទិវាពលកម្មអន្តរជាតិ"),
            ("2022-05-14", "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម ព្រះករុណា ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី"),
            ("2022-05-15", "ពិធីបុណ្យវិសាខបូជា"),
            ("2022-05-19", "ព្រះរាជពិធីច្រត់ព្រះនង្គ័ល"),
            ("2022-06-18", "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម សម្តេចព្រះមហាក្សត្រី ព្រះវររាជមាតា នរោត្តម មុនិនាថ សីហនុ"),
            ("2022-09-24", "ទិវាប្រកាសរដ្ឋធម្មនុញ្ញ; ពិធីបុណ្យភ្ផុំបិណ្ឌ"),
            ("2022-09-25", "ពិធីបុណ្យភ្ផុំបិណ្ឌ"),
            ("2022-09-26", "ពិធីបុណ្យភ្ផុំបិណ្ឌ"),
            (
                "2022-10-15",
                "ទិវាប្រារព្ឋពិធីគោរពព្រះវិញ្ញាណក្ខន្ឋ ព្រះករុណា ព្រះបាទសម្តេចព្រះ នរោត្តម "
                "សីហនុ ព្រះមហាវីរក្សត្រ ព្រះវររាជបិតាឯករាជ្យ បូរណភាពទឹកដី "
                "និងឯកភាពជាតិខ្មែរ ព្រះបរមរតនកោដ្ឋ",
            ),
            (
                "2022-10-29",
                "ព្រះរាជពិធីគ្រងព្រះបរមរាជសម្បត្តិ របស់ ព្រះករុណា "
                "ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី "
                "ព្រះមហាក្សត្រនៃព្រះរាជាណាចក្រកម្ពុជា",
            ),
            ("2022-11-07", "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
            ("2022-11-08", "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
            ("2022-11-09", "ពិធីបុណ្យឯករាជ្យជាតិ; ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
        )

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "ទិវាចូលឆ្នាំសាកល"),
            ("2023-01-07", "ទិវាជ័យជម្នះលើរបបប្រល័យពូជសាសន៍"),
            ("2023-03-08", "ទិវាអន្តរជាតិនារី"),
            ("2023-04-14", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2023-04-15", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2023-04-16", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2023-05-01", "ទិវាពលកម្មអន្តរជាតិ"),
            ("2023-05-04", "ពិធីបុណ្យវិសាខបូជា"),
            ("2023-05-08", "ព្រះរាជពិធីច្រត់ព្រះនង្គ័ល"),
            ("2023-05-14", "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម ព្រះករុណា ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី"),
            ("2023-06-18", "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម សម្តេចព្រះមហាក្សត្រី ព្រះវររាជមាតា នរោត្តម មុនិនាថ សីហនុ"),
            ("2023-09-24", "ទិវាប្រកាសរដ្ឋធម្មនុញ្ញ"),
            ("2023-10-13", "ពិធីបុណ្យភ្ផុំបិណ្ឌ"),
            ("2023-10-14", "ពិធីបុណ្យភ្ផុំបិណ្ឌ"),
            (
                "2023-10-15",
                "ទិវាប្រារព្ឋពិធីគោរពព្រះវិញ្ញាណក្ខន្ឋ ព្រះករុណា ព្រះបាទសម្តេចព្រះ នរោត្តម "
                "សីហនុ ព្រះមហាវីរក្សត្រ ព្រះវររាជបិតាឯករាជ្យ បូរណភាពទឹកដី "
                "និងឯកភាពជាតិខ្មែរ ព្រះបរមរតនកោដ្ឋ; ពិធីបុណ្យភ្ផុំបិណ្ឌ",
            ),
            (
                "2023-10-29",
                "ព្រះរាជពិធីគ្រងព្រះបរមរាជសម្បត្តិ របស់ ព្រះករុណា "
                "ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី "
                "ព្រះមហាក្សត្រនៃព្រះរាជាណាចក្រកម្ពុជា",
            ),
            ("2023-11-09", "ពិធីបុណ្យឯករាជ្យជាតិ"),
            ("2023-11-26", "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
            ("2023-11-27", "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
            ("2023-11-28", "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
        )

    def test_new_years_day(self):
        self.assertHolidayName("ទិវាចូលឆ្នាំសាកល", (f"{year}-01-01" for year in self.full_range))

    def test_day_of_victory_over_genocidal_regime(self):
        self.assertHolidayName(
            "ទិវាជ័យជម្នះលើរបបប្រល័យពូជសាសន៍", (f"{year}-01-07" for year in self.full_range)
        )

    def test_international_womens_day(self):
        self.assertHolidayName("ទិវាអន្តរជាតិនារី", (f"{year}-03-08" for year in self.full_range))

    def test_khmer_new_year(self):
        name = "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"
        years_sangkranta_apr14 = {
            2017,
            2018,
            2021,
            2022,
            2023,
            2025,
            2026,
            2027,
            2029,
            2030,
            2031,
        }
        self.assertHolidayNameCount(
            name, 3, range(self.start_year, 2020), range(2021, 2024), range(2025, self.end_year)
        )
        self.assertHolidayNameCount(name, 4, 2024)
        self.assertHolidayName(name, (f"{year}-04-14" for year in years_sangkranta_apr14))
        self.assertHolidayName(
            name,
            (
                f"{year}-04-13"
                for year in self.full_range
                if year not in years_sangkranta_apr14 and year != 2020
            ),
        )
        self.assertNoHolidayName(name, 2020)
        self.assertNoHolidayName(name, (f"{year}-04-13" for year in years_sangkranta_apr14))
        self.assertNoHolidayName(
            name,
            (
                f"{year}-04-16"
                for year in self.full_range
                if year not in years_sangkranta_apr14 and year not in {2020, 2024}
            ),
        )

    def test_international_labor_day(self):
        self.assertHolidayName("ទិវាពលកម្មអន្តរជាតិ", (f"{year}-05-01" for year in self.full_range))

    def test_king_sihamoni_birthday(self):
        name = "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម ព្រះករុណា ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី"
        self.assertHolidayName(
            name,
            (f"{year}-05-13" for year in range(2005, 2020)),
            (f"{year}-05-14" for year in range(2005, self.end_year)),
            (f"{year}-05-15" for year in range(2005, 2020)),
        )
        self.assertNoHolidayName(name, range(self.start_year, 2005))
        self.assertHolidayNameCount(name, 1, range(2020, self.end_year))

    def test_national_day_of_remembrance(self):
        name = "ទិវាជាតិនៃការចងចាំ"
        self.assertHolidayName(name, (f"{year}-05-20" for year in range(2018, 2020)))
        self.assertNoHolidayName(name, range(self.start_year, 2018), range(2020, self.end_year))

    def test_international_children_day(self):
        name = "ទិវាកុមារអន្តរជាតិ"
        self.assertHolidayName(name, (f"{year}-06-01" for year in range(self.start_year, 2020)))
        self.assertNoHolidayName(name, range(2020, self.end_year))

    def test_queen_mother_monineath_birthday(self):
        name = "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម សម្តេចព្រះមហាក្សត្រី ព្រះវររាជមាតា នរោត្តម មុនិនាថ សីហនុ"
        self.assertHolidayName(name, (f"{year}-06-18" for year in range(1994, self.end_year)))
        self.assertNoHolidayName(name, self.start_year)

    def test_constitution_day(self):
        self.assertHolidayName("ទិវាប្រកាសរដ្ឋធម្មនុញ្ញ", (f"{year}-09-24" for year in self.full_range))

    def test_king_sihanouk_memorial_day(self):
        name = (
            "ទិវាប្រារព្ឋពិធីគោរពព្រះវិញ្ញាណក្ខន្ឋ ព្រះករុណា ព្រះបាទសម្តេចព្រះ"
            " នរោត្តម សីហនុ ព្រះមហាវីរក្សត្រ ព្រះវររាជបិតាឯករាជ្យ បូរណភាពទឹកដី"
            " និងឯកភាពជាតិខ្មែរ ព្រះបរមរតនកោដ្ឋ"
        )
        self.assertHolidayName(name, (f"{year}-10-15" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_paris_peace_agreement_day(self):
        name = "ទិវារំលឹកសន្ធិសញ្ញាសន្តិភាពទីក្រុងប៉ារីស"
        self.assertHolidayName(name, (f"{year}-10-23" for year in range(self.start_year, 2020)))
        self.assertNoHolidayName(name, range(2020, self.end_year))

    def test_king_sihamoni_coronation_day(self):
        name = (
            "ព្រះរាជពិធីគ្រងព្រះបរមរាជសម្បត្តិ របស់ ព្រះករុណា "
            "ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី "
            "ព្រះមហាក្សត្រនៃព្រះរាជាណាចក្រកម្ពុជា"
        )
        self.assertHolidayName(name, (f"{year}-10-29" for year in range(2004, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2004))

    def test_national_independence_day(self):
        self.assertHolidayName("ពិធីបុណ្យឯករាជ្យជាតិ", (f"{year}-11-09" for year in self.full_range))

    def test_international_human_rights_day(self):
        name = "ទិវាសិទ្ធិមនុស្សអន្តរជាតិ"
        self.assertHolidayName(name, (f"{year}-12-10" for year in range(self.start_year, 2020)))
        self.assertNoHolidayName(name, range(2020, self.end_year))

    def test_peace_day_in_cambodia(self):
        name = "ទិវាសន្តិភាពនៅកម្ពុជា"
        self.assertHolidayName(name, (f"{year}-12-29" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))

    def test_meak_bochea(self):
        name = "ពិធីបុណ្យមាឃបូជា"
        self.assertHolidayName(
            name,
            "2014-02-14",
            "2015-02-03",
            "2016-02-22",
            "2017-02-11",
            "2018-01-31",
            "2019-02-19",
        )
        self.assertHolidayName(name, range(self.start_year, 2020))
        self.assertNoHolidayName(name, range(2020, self.end_year))

    def test_visaka_bochea(self):
        name = "ពិធីបុណ្យវិសាខបូជា"
        self.assertHolidayName(
            name,
            "2020-05-06",
            "2021-04-26",
            "2022-05-15",
            "2023-05-04",
            "2024-05-22",
            "2025-05-11",
        )
        self.assertHolidayName(name, self.full_range)

    def test_royal_ploughing_ceremony(self):
        name = "ព្រះរាជពិធីច្រត់ព្រះនង្គ័ល"
        self.assertHolidayName(
            name,
            "2020-05-10",
            "2021-04-30",
            "2022-05-19",
            "2023-05-08",
            "2024-05-26",
            "2025-05-15",
        )
        self.assertHolidayName(name, self.full_range)

    def test_pchum_ben(self):
        name = "ពិធីបុណ្យភ្ផុំបិណ្ឌ"
        self.assertHolidayName(
            name,
            # 2 Days Celebration
            "2015-10-11",
            "2015-10-12",
            "2016-09-30",
            "2016-10-01",
            # 3 Days Celebration
            "2017-09-19",
            "2017-09-20",
            "2017-09-21",
            "2018-10-08",
            "2018-10-09",
            "2018-10-10",
            "2019-09-27",
            "2019-09-28",
            "2019-09-29",
            "2020-09-16",
            "2020-09-17",
            "2020-09-18",
            "2021-10-05",
            "2021-10-06",
            "2021-10-07",
            "2022-09-24",
            "2022-09-25",
            "2022-09-26",
            "2023-10-13",
            "2023-10-14",
            "2023-10-15",
        )
        self.assertHolidayNameCount(name, 2, range(self.start_year, 2017))
        self.assertHolidayNameCount(name, 3, range(2017, self.end_year))

    def test_bon_om_touk(self):
        name = "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"
        self.assertHolidayName(
            name,
            # 3 Days Celebration
            "2015-11-24",
            "2015-11-25",
            "2015-11-26",
            "2016-11-13",
            "2016-11-14",
            "2016-11-15",
            "2017-11-02",
            "2017-11-03",
            "2017-11-04",
            "2018-11-21",
            "2018-11-22",
            "2018-11-23",
            "2019-11-10",
            "2019-11-11",
            "2019-11-12",
            "2020-10-30",
            "2020-10-31",
            "2020-11-01",
            "2021-11-18",
            "2021-11-19",
            "2021-11-20",
            "2022-11-07",
            "2022-11-08",
            "2022-11-09",
            "2023-11-26",
            "2023-11-27",
            "2023-11-28",
        )
        self.assertHolidayNameCount(name, 3, self.full_range)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "ទិវាចូលឆ្នាំសាកល"),
            ("2023-01-07", "ទិវាជ័យជម្នះលើរបបប្រល័យពូជសាសន៍"),
            ("2023-03-08", "ទិវាអន្តរជាតិនារី"),
            ("2023-04-14", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2023-04-15", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2023-04-16", "ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"),
            ("2023-05-01", "ទិវាពលកម្មអន្តរជាតិ"),
            ("2023-05-04", "ពិធីបុណ្យវិសាខបូជា"),
            ("2023-05-08", "ព្រះរាជពិធីច្រត់ព្រះនង្គ័ល"),
            ("2023-05-14", "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម ព្រះករុណា ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី"),
            ("2023-06-18", "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម សម្តេចព្រះមហាក្សត្រី ព្រះវររាជមាតា នរោត្តម មុនិនាថ សីហនុ"),
            ("2023-09-24", "ទិវាប្រកាសរដ្ឋធម្មនុញ្ញ"),
            ("2023-10-13", "ពិធីបុណ្យភ្ផុំបិណ្ឌ"),
            ("2023-10-14", "ពិធីបុណ្យភ្ផុំបិណ្ឌ"),
            (
                "2023-10-15",
                "ទិវាប្រារព្ឋពិធីគោរពព្រះវិញ្ញាណក្ខន្ឋ ព្រះករុណា ព្រះបាទសម្តេចព្រះ នរោត្តម "
                "សីហនុ ព្រះមហាវីរក្សត្រ ព្រះវររាជបិតាឯករាជ្យ បូរណភាពទឹកដី "
                "និងឯកភាពជាតិខ្មែរ ព្រះបរមរតនកោដ្ឋ; ពិធីបុណ្យភ្ផុំបិណ្ឌ",
            ),
            (
                "2023-10-29",
                "ព្រះរាជពិធីគ្រងព្រះបរមរាជសម្បត្តិ របស់ ព្រះករុណា "
                "ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី "
                "ព្រះមហាក្សត្រនៃព្រះរាជាណាចក្រកម្ពុជា",
            ),
            ("2023-11-09", "ពិធីបុណ្យឯករាជ្យជាតិ"),
            ("2023-11-26", "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
            ("2023-11-27", "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
            ("2023-11-28", "ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "International New Year Day"),
            ("2023-01-07", "Day of Victory over the Genocidal Regime"),
            ("2023-03-08", "International Women's Rights Day"),
            ("2023-04-14", "Khmer New Year's Day"),
            ("2023-04-15", "Khmer New Year's Day"),
            ("2023-04-16", "Khmer New Year's Day"),
            ("2023-05-01", "International Labor Day"),
            ("2023-05-04", "Visaka Bochea Day"),
            ("2023-05-08", "Royal Ploughing Ceremony"),
            ("2023-05-14", "HM King Norodom Sihamoni's Birthday"),
            ("2023-06-18", "HM Queen Norodom Monineath Sihanouk the Queen-Mother's Birthday"),
            ("2023-09-24", "Constitution Day"),
            ("2023-10-13", "Pchum Ben Day"),
            ("2023-10-14", "Pchum Ben Day"),
            ("2023-10-15", "HM King Norodom Sihanouk Mourning Day; Pchum Ben Day"),
            ("2023-10-29", "HM King Norodom Sihamoni's Coronation Day"),
            ("2023-11-09", "National Independence Day"),
            ("2023-11-26", "Water Festival"),
            ("2023-11-27", "Water Festival"),
            ("2023-11-28", "Water Festival"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2023-01-01", "วันปีใหม่สากล"),
            ("2023-01-07", "วันชัยชนะเหนือระบอบฆ่าล้างเผ่าพันธุ์เขมรแดง"),
            ("2023-03-08", "วันสตรีสากล"),
            ("2023-04-14", "เทศกาลขึ้นปีใหม่ประเพณี"),
            ("2023-04-15", "เทศกาลขึ้นปีใหม่ประเพณี"),
            ("2023-04-16", "เทศกาลขึ้นปีใหม่ประเพณี"),
            ("2023-05-01", "วันแรงงานสากล"),
            ("2023-05-04", "วันวิสาขบูชา"),
            ("2023-05-08", "พระราชพิธีบุญจรดพระนังคัลแรกนาขวัญ"),
            (
                "2023-05-14",
                "พระราชพิธีเฉลิมพระชนมพรรษา พระบาทสมเด็จพระบรมนาถ นโรดมสีหมุนี "
                "พระมหากษัตริย์แห่งราชอาณาจักรกัมพูชา",
            ),
            ("2023-06-18", "พระราชพิธีเฉลิมพระชนมพรรษา สมเด็จพระบรมราชินี นโรดม มนีนาถ สีหนุ"),
            ("2023-09-24", "วันรัฐธรรมนูญ"),
            ("2023-10-13", "เทศกาลงานวันสาร์ทภจุมบิณฑ์เขมร"),
            ("2023-10-14", "เทศกาลงานวันสาร์ทภจุมบิณฑ์เขมร"),
            (
                "2023-10-15",
                "วันสดุดีพระบาทสมเด็จพระบรมนาถนโรดม สีหนุ พระบิดาแห่งเอกราช "
                "บูรณภาพแห่งดินแดน และเอกภาพของชาติกัมพูชา; เทศกาลงานวันสาร์ทภจุมบิณฑ์เขมร",
            ),
            (
                "2023-10-29",
                "พระราชพิธีเฉลิมฉลองการขึ้นครองราชสมบัติ พระบาทสมเด็จพระบรมนาถ นโรดมสีหมุนี "
                "พระมหากษัตริย์แห่งราชอาณาจักรกัมพูชา",
            ),
            ("2023-11-09", "วันประกาศเอกราชจากฝรั่งเศส"),
            ("2023-11-26", "พระราชพิธีบุญแข่งเรือลอยกระทงไฟไหว้พระจันทร์และกินข้าวเม่า"),
            ("2023-11-27", "พระราชพิธีบุญแข่งเรือลอยกระทงไฟไหว้พระจันทร์และกินข้าวเม่า"),
            ("2023-11-28", "พระราชพิธีบุญแข่งเรือลอยกระทงไฟไหว้พระจันทร์และกินข้าวเม่า"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.marshall_islands import MarshallIslands
from tests.common import CommonCountryTests


class TestMarshallIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(MarshallIslands)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2010-12-31",
            "2012-01-02",
            "2017-01-02",
            "2021-12-31",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} Holiday", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_nuclear_victims_remembrance_day(self):
        name_1996 = "Nuclear Victims Remembrance Day"
        name_2003 = "Memorial Day and Nuclear Survivors Remembrance Day"
        self.assertHolidayName(
            name_1996,
            (f"{year}-03-01" for year in (*range(1996, 2004), *range(2006, self.end_year))),
        )
        self.assertHolidayName(name_2003, (f"{year}-03-01" for year in range(2004, 2006)))
        self.assertNoHolidayName(name_1996, range(self.start_year, 1996), range(2004, 2006))
        self.assertNoHolidayName(
            name_2003, range(self.start_year, 2004), range(2006, self.end_year)
        )
        obs_dts = (
            "2014-02-28",
            "2015-03-02",
            "2020-03-02",
            "2025-02-28",
            "2026-03-02",
        )
        self.assertHolidayName(f"{name_1996} Holiday", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(2006, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2006))
        obs_dts = (
            "2010-04-30",
            "2011-05-02",
            "2016-05-02",
            "2021-04-30",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} Holiday", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_fishermans_day(self):
        name = "Fisherman's Day"
        self.assertHolidayName(
            name,
            "2020-07-03",
            "2021-07-02",
            "2022-07-01",
            "2023-07-07",
            "2024-07-05",
            "2025-07-04",
        )
        self.assertHolidayName(name, range(1996, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1996))

    def test_dri_jerbal_day(self):
        name = "Dri-jerbal Day"
        self.assertHolidayName(
            name,
            "2020-09-04",
            "2021-09-03",
            "2022-09-02",
            "2023-09-01",
            "2024-09-06",
            "2025-09-05",
        )
        self.assertHolidayName(name, range(1996, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1996))

    def test_manit_day(self):
        name = "Manit Day"
        self.assertHolidayName(
            name,
            "2020-09-25",
            "2021-09-24",
            "2022-09-30",
            "2023-09-29",
            "2024-09-27",
            "2025-09-26",
        )
        self.assertHolidayName(name, range(1996, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1996))

    def test_presidents_day(self):
        name = "President's Day"
        self.assertHolidayName(name, (f"{year}-11-17" for year in self.full_range))
        obs_dts = (
            "2012-11-16",
            "2013-11-18",
            "2018-11-16",
            "2019-11-18",
            "2024-11-18",
        )
        self.assertHolidayName(f"{name} Holiday", obs_dts)

    def test_general_election_day(self):
        name = "General Election Day"
        self.assertHolidayName(
            name,
            "2015-11-16",
            "2019-11-18",
            "2023-11-20",
        )
        self.assertHolidayName(
            name, (year for year in range(2015, self.end_year) if year % 4 == 3)
        )
        self.assertNoHolidayName(
            name,
            range(self.start_year, 2015),
            (year for year in range(2015, self.end_year) if year % 4 != 3),
        )

    def test_gospel_day(self):
        name = "Gospel Day"
        self.assertHolidayName(
            name,
            "2020-12-04",
            "2021-12-03",
            "2022-12-02",
            "2023-12-01",
            "2024-12-06",
            "2025-12-05",
        )
        self.assertHolidayName(name, self.full_range)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-24",
            "2011-12-26",
            "2016-12-26",
            "2021-12-24",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} Holiday", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-03-01", "Nuclear Victims Remembrance Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-05-01", "Constitution Day"),
            ("2024-07-05", "Fisherman's Day"),
            ("2024-09-06", "Dri-jerbal Day"),
            ("2024-09-27", "Manit Day"),
            ("2024-11-17", "President's Day"),
            ("2024-11-18", "President's Day Holiday"),
            ("2024-12-06", "Gospel Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-02-28", "Nuclear Victims Remembrance Day Holiday"),
            ("2025-03-01", "Nuclear Victims Remembrance Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-05-01", "Constitution Day"),
            ("2025-07-04", "Fisherman's Day"),
            ("2025-09-05", "Dri-jerbal Day"),
            ("2025-09-26", "Manit Day"),
            ("2025-11-17", "President's Day"),
            ("2025-12-05", "Gospel Day"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.senegal import Senegal
from tests.common import CommonCountryTests


class TestSenegal(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1964, 2050)
        super().setUpClass(Senegal, years=years, years_non_observed=years)
        cls.no_estimated_holidays = Senegal(years=years, islamic_show_estimated=False)

    def test_special_holidays(self):
        self.assertHoliday(
            "2018-10-29",
            "2022-12-26",
        )

    def test_new_years_day(self):
        self.assertHolidayName("Jour de l'an", (f"{year}-01-01" for year in range(1964, 2050)))

    def test_confederation_day(self):
        name = "Fête de la Confédération de la Sénégambie"
        self.assertHolidayName(name, (f"{year}-02-01" for year in range(1983, 1990)))
        self.assertNoHolidayName(name, range(1964, 1983), range(1990, 2050))

    def test_independence_day(self):
        self.assertHolidayName(
            "Fête de l'Indépendance",
            (f"{year}-07-14" for year in range(1964, 1975)),
            (f"{year}-04-04" for year in range(1975, 2050)),
        )

    def test_labor_day(self):
        self.assertHolidayName("Fête du Travail", (f"{year}-05-01" for year in range(1964, 2050)))

    def test_assumption_of_mary_day(self):
        self.assertHolidayName("Assomption", (f"{year}-08-15" for year in range(1964, 2050)))

    def test_all_saints_day(self):
        self.assertHolidayName("Toussaint", (f"{year}-11-01" for year in range(1964, 2050)))

    def test_christmas_day(self):
        self.assertHolidayName("Noël", (f"{year}-12-25" for year in range(1964, 2050)))

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        dts = (
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(1964, 2050))

    def test_ascension_day(self):
        name = "Jeudi de l'Ascension"
        dts = (
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(1964, 2050))

    def test_whit_monday(self):
        name = "Lundi de Pentecôte"
        dts = (
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(1964, 2050))

    def test_ashura(self):
        name = "Tamxarit"
        dts = (
            "2020-08-29",
            "2021-08-18",
            "2022-08-08",
            "2023-07-28",
            "2024-07-17",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.no_estimated_holidays, range(1983, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(1964, 1983))

    def test_grand_magal_of_touba(self):
        name = "Grand Magal de Touba"
        dts = (
            "2020-10-06",
            "2021-09-26",
            "2022-09-15",
            "2023-09-04",
            "2024-08-23",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.no_estimated_holidays, range(2014, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(1964, 2014))

    def test_mawlid_day(self):
        name = "Journée du Maouloud"
        dts = (
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.no_estimated_holidays, range(1964, 2050))

    def test_eid_al_fitr(self):
        name = "Journée de la Korité"
        dts = (
            "2020-05-24",
            "2021-05-12",
            "2022-05-01",
            "2023-04-22",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.no_estimated_holidays, range(1964, 2050))
        obs_dt = (
            "2020-05-25",
            "2022-05-02",
            "2025-03-31",
        )
        self.assertHolidayName(f"{name} (observé)", self.no_estimated_holidays, obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_eid_al_adha(self):
        name = "Journée de la Tabaski"
        dts = (
            "2020-07-31",
            "2021-07-21",
            "2022-07-10",
            "2023-06-29",
            "2024-06-17",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.no_estimated_holidays, range(1964, 2050))
        obs_dt = (
            "2016-09-12",
            "2019-08-12",
            "2022-07-11",
        )
        self.assertHolidayName(f"{name} (observé)", self.no_estimated_holidays, obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Jour de l'an"),
            ("2024-04-01", "Lundi de Pâques"),
            ("2024-04-04", "Fête de l'Indépendance"),
            ("2024-04-10", "Journée de la Korité"),
            ("2024-05-01", "Fête du Travail"),
            ("2024-05-09", "Jeudi de l'Ascension"),
            ("2024-05-20", "Lundi de Pentecôte"),
            ("2024-06-17", "Journée de la Tabaski"),
            ("2024-07-17", "Tamxarit"),
            ("2024-08-15", "Assomption"),
            ("2024-08-23", "Grand Magal de Touba"),
            ("2024-09-15", "Journée du Maouloud"),
            ("2024-11-01", "Toussaint"),
            ("2024-12-25", "Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-04", "Independence Day"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-09", "Ascension Day"),
            ("2024-05-20", "Whit Monday"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-07-17", "Ashura"),
            ("2024-08-15", "Assumption Day"),
            ("2024-08-23", "Grand Magal of Touba"),
            ("2024-09-15", "Prophet's Birthday"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.uganda import Uganda
from tests.common import CommonCountryTests


class TestUganda(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1963, 2050)
        super().setUpClass(Uganda, years=years)
        cls.no_estimated_holidays = Uganda(years=years, islamic_show_estimated=False)

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in range(1963, 2050)))

    def test_nrm_liberation_day(self):
        name = "NRM Liberation Day"
        self.assertHolidayName(name, (f"{year}-01-26" for year in range(1987, 2050)))
        self.assertNoHolidayName(name, range(1963, 1987))

    def test_archbishop_janani_luwum_day(self):
        name = "Archbishop Janani Luwum Day"
        self.assertHolidayName(name, (f"{year}-02-16" for year in range(2016, 2050)))
        self.assertNoHolidayName(name, range(1963, 2016))

    def test_womens_day(self):
        self.assertHolidayName("Women's Day", (f"{year}-03-08" for year in range(1963, 2050)))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1963, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1963, 2050))

    def test_labour_day(self):
        self.assertHolidayName("Labour Day", (f"{year}-05-01" for year in range(1963, 2050)))

    def test_uganda_martyrs_day(self):
        self.assertHolidayName(
            "Uganda Martyrs' Day", (f"{year}-06-03" for year in range(1963, 2050))
        )

    def test_national_heroes_day(self):
        name = "National Heroes' Day"
        self.assertHolidayName(name, (f"{year}-06-09" for year in range(2001, 2050)))
        self.assertNoHolidayName(name, range(1963, 2001))

    def test_independence_day(self):
        self.assertHolidayName("Independence Day", (f"{year}-10-09" for year in range(1963, 2050)))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in range(1963, 2050)))

    def test_boxing_day(self):
        self.assertHolidayName("Boxing Day", (f"{year}-12-26" for year in range(1963, 2050)))

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertHolidayName(
            name,
            self.no_estimated_holidays,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1963, 2050))

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertHolidayName(
            name,
            self.no_estimated_holidays,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1963, 2050))

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-01-26", "NRM Liberation Day"),
            ("2020-02-16", "Archbishop Janani Luwum Day"),
            ("2020-03-08", "Women's Day"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-05-01", "Labour Day"),
            ("2020-05-24", "Eid al-Fitr (estimated)"),
            ("2020-06-03", "Uganda Martyrs' Day"),
            ("2020-06-09", "National Heroes' Day"),
            ("2020-07-31", "Eid al-Adha (estimated)"),
            ("2020-10-09", "Independence Day"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cabo_verde import CaboVerde
from tests.common import CommonCountryTests


class TestCapeVerde(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(CaboVerde)

    def test_new_years_day(self):
        self.assertHolidayName("Ano Novo", (f"{year}-01-01" for year in self.full_range))

    def test_democracy_and_freedom_day(self):
        name = "Dia da Liberdade e da Democracia"
        self.assertHolidayName(name, (f"{year}-01-13" for year in range(2000, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2000))

    def test_heroes_day(self):
        self.assertHolidayName(
            "Dia da Nacionalidade e dos Heróis Nacionais",
            (f"{year}-01-20" for year in self.full_range),
        )

    def test_ash_wednesday(self):
        name = "Quarta-feira de Cinzas"
        self.assertHolidayName(
            name,
            "2020-02-26",
            "2021-02-17",
            "2022-03-02",
            "2023-02-22",
            "2024-02-14",
            "2025-03-05",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Sexta-feira Santa"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Páscoa"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_workers_day(self):
        self.assertHolidayName("Dia do Trabalhador", (f"{year}-05-01" for year in self.full_range))

    def test_international_childrens_day(self):
        name = "Dia Mundial da Criança"
        self.assertHolidayName(name, (f"{year}-06-01" for year in range(2005, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2005))

    def test_independence_day(self):
        self.assertHolidayName(
            "Dia da Independência Nacional", (f"{year}-07-05" for year in self.full_range)
        )

    def test_assumption_day(self):
        self.assertHolidayName("Dia da Assunção", (f"{year}-08-15" for year in self.full_range))

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Dia de Todos os Santos", (f"{year}-11-01" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("Dia do Natal", (f"{year}-12-25" for year in self.full_range))

    def test_holy_thursday(self):
        name = "Quinta-Feira Santa"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_mothers_day(self):
        name = "Dia das Mães"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-05-10",
            "2021-05-09",
            "2022-05-08",
            "2023-05-14",
            "2024-05-12",
            "2025-05-11",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_fathers_day(self):
        name = "Dia dos Pais"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-06-21",
            "2021-06-20",
            "2022-06-19",
            "2023-06-18",
            "2024-06-16",
            "2025-06-15",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_brava_municipality_day(self):
        name = "Dia do Município da Brava"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BR":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-24" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_boa_vista_municipality_day(self):
        name = "Dia do Município da Boa Vista"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BV":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-04" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_santa_catarina_de_santiago_municipality_day(self):
        name = "Dia do Município de Santa Catarina de Santiago"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "CA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-25" for year in range(1982, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1982))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_santa_catarina_do_fogo_municipality_day(self):
        name = "Dia do Município de Santa Catarina do Fogo"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "CF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-25" for year in range(2005, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2005))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_santa_cruz_municipality_day(self):
        name = "Dia do Município de Santa Cruz"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "CR":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-25" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_maio_municipality_day(self):
        name = "Dia do Município do Maio"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "MA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-08" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_mosteiros_municipality_day(self):
        name = "Dia do Município dos Mosteiros"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "MO":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-15" for year in range(1992, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1992))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_santo_antao_island_day(self):
        name = "Dia da Ilha de Santo Antão"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"PA", "PN", "RG"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-17" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_paul_municipality_day(self):
        name = "Dia do Município do Paúl"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-13" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_porto_novo_municipality_day(self):
        name = "Dia do Município do Porto Novo"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PN":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-02" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_praia_municipality_day(self):
        name = "Dia do Município da Praia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PR":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-19" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_ribeira_brava_municipality_day(self):
        name = "Dia do Município de Ribeira Brava"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "RB":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-06" for year in range(2005, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2005))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_ribeira_grande_municipality_day(self):
        name = "Dia do Município de Ribeira Grande"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "RG":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-07" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_ribeira_grande_de_santiago_municipality_day(self):
        name = "Dia do Município de Ribeira Grande de Santiago"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "RS":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-31" for year in range(2006, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2006))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sao_domingos_municipality_day(self):
        name = "Dia do Município de São Domingos"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SD":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-13" for year in range(1994, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1994))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sao_filipe_municipality_day(self):
        name = "Dia do Município de São Filipe"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-01" for year in range(1992, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1992))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sal_municipality_day(self):
        name = "Dia do Município do Sal"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SL":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-15" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sao_miguel_municipality_day(self):
        name = "Dia do Município de São Miguel"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SM":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-29" for year in range(1997, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1997))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sao_lourenco_dos_orgaos_municipality_day(self):
        name = "Dia do Município de São Lourenço dos Órgãos"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SO":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-09" for year in range(2005, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2005))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sao_salvador_do_mundo_municipality_day(self):
        name = "Dia do Município de São Salvador do Mundo"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SS":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-19" for year in range(2005, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2005))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sao_vicente_municipality_day(self):
        name = "Dia do Município de São Vicente"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SV":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-22" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_carnival_tuesday(self):
        name = "Terça-feira de Carnaval"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SV":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-02-25",
                    "2021-02-16",
                    "2022-03-01",
                    "2023-02-21",
                    "2024-02-13",
                    "2025-03-04",
                )
                self.assertHolidayName(name, holidays, range(1983, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_tarrafal_de_santiago_municipality_day(self):
        name = "Dia do Município do Tarrafal de Santiago"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-15" for year in range(1983, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1983))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_tarrafal_de_sao_nicolau_municipality_day(self):
        name = "Dia do Município do Tarrafal de São Nicolau"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TS":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-02" for year in range(2005, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2005))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_2024_public_holidays(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Ano Novo"),
            ("2024-01-13", "Dia da Liberdade e da Democracia"),
            ("2024-01-20", "Dia da Nacionalidade e dos Heróis Nacionais"),
            ("2024-02-14", "Quarta-feira de Cinzas"),
            ("2024-03-29", "Sexta-feira Santa"),
            ("2024-03-31", "Páscoa"),
            ("2024-05-01", "Dia do Trabalhador"),
            ("2024-06-01", "Dia Mundial da Criança"),
            ("2024-07-05", "Dia da Independência Nacional"),
            ("2024-08-15", "Dia da Assunção"),
            ("2024-11-01", "Dia de Todos os Santos"),
            ("2024-12-25", "Dia do Natal"),
        )

    def test_2024_optional_holidays(self):
        self.assertOptionalHolidaysInYear(
            2024,
            ("2024-03-28", "Quinta-Feira Santa"),
            ("2024-05-12", "Dia das Mães"),
            ("2024-06-16", "Dia dos Pais"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Ano Novo"),
            ("2025-01-13", "Dia da Liberdade e da Democracia"),
            ("2025-01-15", "Dia do Município do Tarrafal de Santiago"),
            ("2025-01-17", "Dia da Ilha de Santo Antão"),
            ("2025-01-20", "Dia da Nacionalidade e dos Heróis Nacionais"),
            ("2025-01-22", "Dia do Município de São Vicente"),
            ("2025-01-31", "Dia do Município de Ribeira Grande de Santiago"),
            ("2025-03-04", "Terça-feira de Carnaval"),
            ("2025-03-05", "Quarta-feira de Cinzas"),
            ("2025-03-13", "Dia do Município de São Domingos"),
            ("2025-04-17", "Quinta-Feira Santa"),
            ("2025-04-18", "Sexta-feira Santa"),
            ("2025-04-20", "Páscoa"),
            ("2025-05-01", "Dia do Município de São Filipe; Dia do Trabalhador"),
            ("2025-05-07", "Dia do Município de Ribeira Grande"),
            ("2025-05-09", "Dia do Município de São Lourenço dos Órgãos"),
            ("2025-05-11", "Dia das Mães"),
            ("2025-05-19", "Dia do Município da Praia"),
            ("2025-06-01", "Dia Mundial da Criança"),
            ("2025-06-13", "Dia do Município do Paúl"),
            ("2025-06-15", "Dia dos Pais"),
            ("2025-06-24", "Dia do Município da Brava"),
            ("2025-07-04", "Dia do Município da Boa Vista"),
            ("2025-07-05", "Dia da Independência Nacional"),
            ("2025-07-19", "Dia do Município de São Salvador do Mundo"),
            ("2025-07-25", "Dia do Município de Santa Cruz"),
            ("2025-08-02", "Dia do Município do Tarrafal de São Nicolau"),
            ("2025-08-15", "Dia da Assunção; Dia do Município dos Mosteiros"),
            ("2025-09-02", "Dia do Município do Porto Novo"),
            ("2025-09-08", "Dia do Município do Maio"),
            ("2025-09-15", "Dia do Município do Sal"),
            ("2025-09-29", "Dia do Município de São Miguel"),
            ("2025-11-01", "Dia de Todos os Santos"),
            (
                "2025-11-25",
                "Dia do Município de Santa Catarina de Santiago; "
                "Dia do Município de Santa Catarina do Fogo",
            ),
            ("2025-12-06", "Dia do Município de Ribeira Brava"),
            ("2025-12-25", "Dia do Natal"),
        )

    def test_l10n_de(self):
        self.assertLocalizedHolidays(
            "de",
            ("2025-01-01", "Neujahr"),
            ("2025-01-13", "Tag der Demokratie und Freiheit"),
            ("2025-01-15", "Tag der Gemeinde Tarrafal de Santiago"),
            ("2025-01-17", "Tag der Insel Santo Antão"),
            ("2025-01-20", "Tag der Nationalhelden"),
            ("2025-01-22", "Tag der Gemeinde São Vicente"),
            ("2025-01-31", "Tag der Gemeinde Ribeira Grande de Santiago"),
            ("2025-03-04", "Faschingsdienstag"),
            ("2025-03-05", "Aschermittwoch"),
            ("2025-03-13", "Tag der Gemeinde São Domingos"),
            ("2025-04-17", "Gründonnerstag"),
            ("2025-04-18", "Karfreitag"),
            ("2025-04-20", "Ostersonntag"),
            ("2025-05-01", "Tag der Arbeit; Tag der Gemeinde São Filipe"),
            ("2025-05-07", "Tag der Gemeinde Ribeira Grande"),
            ("2025-05-09", "Tag der Gemeinde São Lourenço dos Órgãos"),
            ("2025-05-11", "Muttertag"),
            ("2025-05-19", "Tag der Gemeinde Praia"),
            ("2025-06-01", "Weltkindertag"),
            ("2025-06-13", "Tag der Gemeinde Paúl"),
            ("2025-06-15", "Vatertag"),
            ("2025-06-24", "Tag der Gemeinde Brava"),
            ("2025-07-04", "Tag der Gemeinde Boa Vista"),
            ("2025-07-05", "Unabhängigkeitstag"),
            ("2025-07-19", "Tag der Gemeinde São Salvador do Mundo"),
            ("2025-07-25", "Tag der Gemeinde Santa Cruz"),
            ("2025-08-02", "Tag der Gemeinde Tarrafal de São Nicolau"),
            ("2025-08-15", "Mariä Himmelfahrt; Tag der Gemeinde Mosteiros"),
            ("2025-09-02", "Tag der Gemeinde Porto Novo"),
            ("2025-09-08", "Tag der Gemeinde Maio"),
            ("2025-09-15", "Tag der Gemeinde Sal"),
            ("2025-09-29", "Tag der Gemeinde São Miguel"),
            ("2025-11-01", "Allerheiligen"),
            (
                "2025-11-25",
                "Tag der Gemeinde Santa Catarina de Santiago; "
                "Tag der Gemeinde Santa Catarina do Fogo",
            ),
            ("2025-12-06", "Tag der Gemeinde Ribeira Brava"),
            ("2025-12-25", "Weihnachten"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-13", "Democracy and Freedom Day"),
            ("2025-01-15", "Tarrafal de Santiago Municipality Day"),
            ("2025-01-17", "Santo Antão Island Day"),
            ("2025-01-20", "National Heroes Day"),
            ("2025-01-22", "São Vicente Municipality Day"),
            ("2025-01-31", "Ribeira Grande de Santiago Municipality Day"),
            ("2025-03-04", "Carnival Tuesday"),
            ("2025-03-05", "Ash Wednesday"),
            ("2025-03-13", "São Domingos Municipality Day"),
            ("2025-04-17", "Holy Thursday"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-20", "Easter Sunday"),
            ("2025-05-01", "São Filipe Municipality Day; Worker's Day"),
            ("2025-05-07", "Ribeira Grande Municipality Day"),
            ("2025-05-09", "São Lourenço dos Órgãos Municipality Day"),
            ("2025-05-11", "Mother's Day"),
            ("2025-05-19", "Praia Municipality Day"),
            ("2025-06-01", "International Children's Day"),
            ("2025-06-13", "Paúl Municipality Day"),
            ("2025-06-15", "Father's Day"),
            ("2025-06-24", "Brava Municipality Day"),
            ("2025-07-04", "Boa Vista Municipality Day"),
            ("2025-07-05", "Independence Day"),
            ("2025-07-19", "São Salvador do Mundo Municipality Day"),
            ("2025-07-25", "Santa Cruz Municipality Day"),
            ("2025-08-02", "Tarrafal de São Nicolau Municipality Day"),
            ("2025-08-15", "Assumption Day; Mosteiros Municipality Day"),
            ("2025-09-02", "Porto Novo Municipality Day"),
            ("2025-09-08", "Maio Municipality Day"),
            ("2025-09-15", "Sal Municipality Day"),
            ("2025-09-29", "São Miguel Municipality Day"),
            ("2025-11-01", "All Saints' Day"),
            (
                "2025-11-25",
                "Santa Catarina de Santiago Municipality Day; "
                "Santa Catarina do Fogo Municipality Day",
            ),
            ("2025-12-06", "Ribeira Brava Municipality Day"),
            ("2025-12-25", "Christmas Day"),
        )

    def test_l10n_es(self):
        self.assertLocalizedHolidays(
            "es",
            ("2025-01-01", "Año Nuevo"),
            ("2025-01-13", "Día de la Libertad y la Democracia"),
            ("2025-01-15", "Día del Municipio de Tarrafal de Santiago"),
            ("2025-01-17", "Día de la Isla de Santo Antão"),
            ("2025-01-20", "Día de la Nacionalidad y de los Héroes Nacionales"),
            ("2025-01-22", "Día del Municipio de São Vicente"),
            ("2025-01-31", "Día del Municipio de Ribeira Grande de Santiago"),
            ("2025-03-04", "Martes de Carnaval"),
            ("2025-03-05", "Miércoles de Ceniza"),
            ("2025-03-13", "Día del Municipio de São Domingos"),
            ("2025-04-17", "Jueves Santo"),
            ("2025-04-18", "Viernes Santo"),
            ("2025-04-20", "Domingo de Pascua"),
            ("2025-05-01", "Día del Municipio de São Filipe; Día del Trabajador"),
            ("2025-05-07", "Día del Municipio de Ribeira Grande"),
            ("2025-05-09", "Día del Municipio de São Lourenço dos Órgãos"),
            ("2025-05-11", "Día de la Madre"),
            ("2025-05-19", "Día del Municipio de Praia"),
            ("2025-06-01", "Día Mundial de la Infancia"),
            ("2025-06-13", "Día del Municipio de Paúl"),
            ("2025-06-15", "Día del Padre"),
            ("2025-06-24", "Día del Municipio de Brava"),
            ("2025-07-04", "Día del Municipio de Boa Vista"),
            ("2025-07-05", "Día de la Independencia Nacional"),
            ("2025-07-19", "Día del Municipio de São Salvador do Mundo"),
            ("2025-07-25", "Día del Municipio de Santa Cruz"),
            ("2025-08-02", "Día del Municipio de Tarrafal de São Nicolau"),
            ("2025-08-15", "Asunción de Nuestra Señora; Día del Municipio de Mosteiros"),
            ("2025-09-02", "Día del Municipio de Porto Novo"),
            ("2025-09-08", "Día del Municipio de Maio"),
            ("2025-09-15", "Día del Municipio de Sal"),
            ("2025-09-29", "Día del Municipio de São Miguel"),
            ("2025-11-01", "Día de Todos los Santos"),
            (
                "2025-11-25",
                "Día del Municipio de Santa Catarina de Santiago; "
                "Día del Municipio de Santa Catarina do Fogo",
            ),
            ("2025-12-06", "Día del Municipio de Ribeira Brava"),
            ("2025-12-25", "Navidad"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2025-01-01", "Nouvel An"),
            ("2025-01-13", "Journée de la liberté et de la démocratie"),
            ("2025-01-15", "Journée de la municipalité de Tarrafal de Santiago"),
            ("2025-01-17", "Journée de l'île de Santo Antão"),
            ("2025-01-20", "Journée de la nationalité et des héros nationaux"),
            ("2025-01-22", "Journée de la municipalité de São Vicente"),
            ("2025-01-31", "Journée de la municipalité de Ribeira Grande de Santiago"),
            ("2025-03-04", "Mardi du Carnaval"),
            ("2025-03-05", "Mercredi des Cendres"),
            ("2025-03-13", "Journée de la municipalité de São Domingos"),
            ("2025-04-17", "Jeudi Saint"),
            ("2025-04-18", "Vendredi Saint"),
            ("2025-04-20", "Dimanche de Pâques"),
            ("2025-05-01", "Fête du travail; Journée de la municipalité de São Filipe"),
            ("2025-05-07", "Journée de la municipalité de Ribeira Grande"),
            ("2025-05-09", "Journée de la municipalité de São Lourenço dos Órgãos"),
            ("2025-05-11", "Fête des Mères"),
            ("2025-05-19", "Journée de la municipalité de Praia"),
            ("2025-06-01", "Journée mondiale de l'enfance"),
            ("2025-06-13", "Journée de la municipalité de Paúl"),
            ("2025-06-15", "Fête des Pères"),
            ("2025-06-24", "Journée de la municipalité de Brava"),
            ("2025-07-04", "Journée de la municipalité de Boa Vista"),
            ("2025-07-05", "Fête de l'Indépendance Nationale"),
            ("2025-07-19", "Journée de la municipalité de São Salvador do Mundo"),
            ("2025-07-25", "Journée de la municipalité de Santa Cruz"),
            ("2025-08-02", "Journée de la municipalité de Tarrafal de São Nicolau"),
            ("2025-08-15", "Assomption de Notre-Dame; Journée de la municipalité des Mosteiros"),
            ("2025-09-02", "Journée de la municipalité de Porto Novo"),
            ("2025-09-08", "Journée de la municipalité de Maio"),
            ("2025-09-15", "Journée de la municipalité de Sal"),
            ("2025-09-29", "Journée de la municipalité de São Miguel"),
            ("2025-11-01", "Toussaint"),
            (
                "2025-11-25",
                "Journée de la municipalité de Santa Catarina de Santiago; "
                "Journée de la municipalité de Santa Catarina do Fogo",
            ),
            ("2025-12-06", "Journée de la municipalité de Ribeira Brava"),
            ("2025-12-25", "Noël"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.madagascar import Madagascar
from tests.common import CommonCountryTests


class TestMadagascar(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Madagascar, years=range(1947, 2050))

    def test_new_years(self):
        self.assertHoliday(f"{year}-01-01" for year in range(1947, 2050))

    def test_womens_day(self):
        self.assertHoliday(f"{year}-03-08" for year in range(1947, 2050))

    def test_martyrs_day(self):
        self.assertHoliday(f"{year}-03-29" for year in range(1947, 2050))

    def test_easter_sunday(self):
        self.assertHoliday(
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
        )

    def test_easter_monday(self):
        self.assertHoliday(
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
        )

    def test_labor_day(self):
        self.assertHoliday(f"{year}-05-01" for year in range(1947, 2050))

    def test_ascension_day(self):
        self.assertHoliday(
            "2019-05-30",
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
        )

    def test_whit_sunday(self):
        self.assertHoliday(
            "2019-06-09",
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
        )

    def test_whit_monday(self):
        self.assertHoliday(
            "2019-06-10",
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
        )

    def test_mothers_day(self):
        self.assertHoliday(
            "2019-05-26",
            "2020-06-07",
            "2021-05-30",
            "2022-05-29",
            "2023-06-04",
        )

    def test_father_day(self):
        self.assertHoliday(
            "2019-06-16",
            "2020-06-21",
            "2021-06-20",
            "2022-06-19",
            "2023-06-18",
        )

    def test_independence_day(self):
        self.assertHoliday(f"{year}-06-26" for year in range(1960, 2050))
        self.assertNoHoliday(f"{year}-06-26" for year in range(1947, 1960))
        self.assertNoHolidayName("Fetin'ny fahaleovantena", range(1947, 1960))

    def test_assumption_day(self):
        self.assertHoliday(f"{year}-08-15" for year in range(1947, 2050))

    def test_all_saints_day(self):
        self.assertHoliday(f"{year}-11-01" for year in range(1947, 2050))

    def test_republic_day(self):
        self.assertHoliday(f"{year}-12-11" for year in range(2011, 2050))
        self.assertNoHoliday(f"{year}-12-11" for year in range(1947, 2011))
        self.assertNoHolidayName("Fetin'ny Repoblika", range(1947, 2011))

    def test_christmas_day(self):
        self.assertHoliday(f"{year}-12-25" for year in range(1947, 2050))

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Taom-baovao"),
            ("2021-03-08", "Fetin'ny vehivavy"),
            ("2021-03-29", "Fetin'ny mahery fo"),
            ("2021-04-04", "Fetin'ny paska"),
            ("2021-04-05", "Alatsinain'ny paska"),
            ("2021-05-01", "Fetin'ny asa"),
            ("2021-05-13", "Fiakaran'ny Jesosy kristy tany an-danitra"),
            ("2021-05-23", "Pentekosta"),
            ("2021-05-24", "Alatsinain'ny pentekosta"),
            ("2021-05-30", "Fetin'ny reny"),
            ("2021-06-20", "Fetin'ny ray"),
            ("2021-06-26", "Fetin'ny fahaleovantena"),
            ("2021-08-15", "Fiakaran'ny Masina Maria tany an-danitra"),
            ("2021-11-01", "Fetin'ny olo-masina"),
            ("2021-12-11", "Fetin'ny Repoblika"),
            ("2021-12-25", "Fetin'ny noely"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Taom-baovao"),
            ("2022-03-08", "Fetin'ny vehivavy"),
            ("2022-03-29", "Fetin'ny mahery fo"),
            ("2022-04-17", "Fetin'ny paska"),
            ("2022-04-18", "Alatsinain'ny paska"),
            ("2022-05-01", "Fetin'ny asa"),
            ("2022-05-26", "Fiakaran'ny Jesosy kristy tany an-danitra"),
            ("2022-05-29", "Fetin'ny reny"),
            ("2022-06-05", "Pentekosta"),
            ("2022-06-06", "Alatsinain'ny pentekosta"),
            ("2022-06-19", "Fetin'ny ray"),
            ("2022-06-26", "Fetin'ny fahaleovantena"),
            ("2022-08-15", "Fiakaran'ny Masina Maria tany an-danitra"),
            ("2022-11-01", "Fetin'ny olo-masina"),
            ("2022-12-11", "Fetin'ny Repoblika"),
            ("2022-12-25", "Fetin'ny noely"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-03-08", "Women's Day"),
            ("2022-03-29", "Martyrs' Day"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-05-29", "Mother's Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-19", "Father's Day"),
            ("2022-06-26", "Independence Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-11", "Republic Day"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-03-08", "Жіночий день"),
            ("2022-03-29", "День мучеників"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-05-29", "День матері"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-19", "День батька"),
            ("2022-06-26", "День незалежності"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-11-01", "День усіх святих"),
            ("2022-12-11", "День Республіки"),
            ("2022-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)


from gettext import gettext as tr

from holidays.calendars.gregorian import SEP
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
    SUN_TO_NEXT_MON,
)


class NorfolkIsland(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Norfolk Island holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Norfolk_Island>
        * <https://web.archive.org/web/20250409233237/https://www.infrastructure.gov.au/territories-regions-cities/territories/norfolk-island/travel-information>
        * [2020](https://web.archive.org/web/20220704121150/http://norfolkislander.com/images/2019_08_30_Gazette_No._38.pdf)
        * [2021](https://web.archive.org/web/20250710234146/http://www.norfolkislander.com/images/2020_07_31_Gazette_No._35.pdf)
        * [2022](https://web.archive.org/web/20250328071352/https://www.nirc.gov.au/files/assets/public/v/1/your-council/documents/nirc-gazettes/2021_07_09_gazette_no_29.pdf)
        * [2023](https://web.archive.org/web/20250328071155/https://www.nirc.gov.au/files/assets/public/v/1/your-council/documents/nirc-gazettes/2022_07_21_gazette_no_29.pdf)
        * [2024](https://web.archive.org/web/20250328070948/https://www.nirc.gov.au/files/assets/public/v/1/your-council/documents/nirc-gazettes/2023_07_13_gazette_no_40.pdf)
        * [2025](https://web.archive.org/web/20250711000525/https://www.nirc.gov.au/files/assets/public/v/1/your-council/documents/nirc-gazettes/2024_07_05_gazette_no_25.pdf)
        * [2026](https://web.archive.org/web/20250713192750/https://www.nirc.gov.au/files/assets/public/v/1/your-council/documents/nirc-gazettes/2025/2025-07-11-gazette-no-26.pdf)
    """

    country = "NF"
    default_language = "en_NF"
    # %s (observed).
    observed_label = tr("%s (observed)")
    supported_languages = ("en_NF", "en_US")
    start_year = 2016

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, NorfolkIslandStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        # Australia Day.
        self._add_observed(self._add_holiday_jan_26(tr("Australia Day")))

        # Foundation Day.
        self._add_observed(self._add_holiday_mar_6(tr("Foundation Day")), rule=SUN_TO_NEXT_MON)

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # ANZAC Day.
        self._add_anzac_day(tr("ANZAC Day"))

        # Bounty Day.
        self._add_observed(bounty_day := self._add_holiday_jun_8(tr("Bounty Day")))

        # Sovereign's Birthday.
        name = (
            # King's Birthday.
            tr("King's Birthday")
            if self._year >= 2023
            # Queen's Birthday.
            else tr("Queen's Birthday")
        )
        # If Sovereign's Birthday falls on the same day as Bounty Day (observed),
        # it is moved to the next Monday.
        if self._is_saturday(bounty_day):
            self._add_holiday_2_days_past_3rd_sat_of_jun(name)
        else:
            self._add_holiday_2_days_past_2nd_sat_of_jun(name)

        # Show Day.
        self._add_holiday_2nd_mon_of_oct(tr("Show Day"))

        # Thanksgiving Day.
        self._add_holiday_last_wed_of_nov(tr("Thanksgiving Day"))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )


class NF(NorfolkIsland):
    pass


class NFK(NorfolkIsland):
    pass


class NorfolkIslandStaticHolidays:
    """Norfolk Island special holidays.

    References:
        * [National Day of Mourning 2022](https://web.archive.org/web/20250711012623/https://www.infrastructure.gov.au/territories-regions-cities/territories/norfolk-island/media-releases/national-day-of-mourning-for-her-majesty-the-queen)
    """

    special_public_holidays = {
        # National Day of Mourning for Queen Elizabeth II.
        2022: (SEP, 22, tr("National Day of Mourning for Queen Elizabeth II")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.austria import Austria
from tests.common import CommonCountryTests


class TestAustria(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Austria, with_subdiv_categories=True)

    def test_new_years_day(self):
        self.assertHolidayName("Neujahr", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany(self):
        name = "Heilige Drei Könige"
        self.assertHolidayName(
            name,
            (
                f"{year}-01-06"
                for year in (*range(self.start_year, 1946), *range(1950, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1946, 1950))

    def test_good_friday(self):
        name = "Karfreitag"
        # PUBLIC.
        self.assertNoHolidayName(name)
        # BANK.
        self.assertBankHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertBankHolidayName(name, self.full_range)
        # PROTESTANT.
        self.assertProtestantHolidayName(
            name,
            "2013-03-29",
            "2014-04-18",
            "2015-04-03",
            "2016-03-25",
            "2017-04-14",
            "2018-03-30",
        )
        self.assertProtestantHolidayName(name, range(1956, 2019))
        self.assertNoProtestantHolidayName(
            name, range(self.start_year, 1956), range(2019, self.end_year)
        )

    def test_easter_monday(self):
        name = "Ostermontag"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Staatsfeiertag"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1946, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1946))

    def test_ascension_day(self):
        name = "Christi Himmelfahrt"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Pfingstmontag"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_corpus_christi(self):
        name = "Fronleichnam"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_saint_peter_and_pauls_day(self):
        name = "Peter und Paul"
        self.assertHolidayName(name, (f"{year}-06-29" for year in range(self.start_year, 1946)))
        self.assertNoHolidayName(name, range(1946, self.end_year))

    def test_assumption_day(self):
        self.assertHolidayName("Mariä Himmelfahrt", (f"{year}-08-15" for year in self.full_range))

    def test_national_day(self):
        name = "Nationalfeiertag"
        self.assertHolidayName(name, (f"{year}-10-26" for year in range(1967, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1967))

    def test_all_saints_day(self):
        self.assertHolidayName("Allerheiligen", (f"{year}-11-01" for year in self.full_range))

    def test_immaculate_conception(self):
        name = "Mariä Empfängnis"
        self.assertHolidayName(
            name,
            (
                f"{year}-12-08"
                for year in (*range(self.start_year, 1945), *range(1955, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1945, 1955))

    def test_christmas_eve(self):
        name = "Heiliger Abend"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Christtag", (f"{year}-12-25" for year in self.full_range))

    def test_saint_stephens_day(self):
        self.assertHolidayName("Stephanstag", (f"{year}-12-26" for year in self.full_range))

    def test_new_years_eve(self):
        name = "Silvester"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_saint_martins_day(self):
        name = "Hl. Martin"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_bank_holidays.items():
            if subdiv == "1":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-11" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_josephs_day(self):
        name = "Hl. Josef"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_bank_holidays.items():
            if subdiv in {"2", "6", "7", "8"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-19" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_1920_carinthian_plebiscite_anniversary(self):
        name = "Tag der Volksabstimmung"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_bank_holidays.items():
            if subdiv == "2":
                self.assertHolidayName(
                    name, holidays, (f"{year}-10-10" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_leopolds_day(self):
        name = "Hl. Leopold"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_bank_holidays.items():
            if subdiv in {"3", "9"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-15" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_florians_day(self):
        name = "Hl. Florian"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_bank_holidays.items():
            if subdiv == "4":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-04" for year in range(2004, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2004))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_ruperts_day(self):
        name = "Hl. Rupert"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_bank_holidays.items():
            if subdiv == "5":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-24" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Neujahr"),
            ("2021-01-06", "Heilige Drei Könige"),
            ("2021-04-05", "Ostermontag"),
            ("2021-05-01", "Staatsfeiertag"),
            ("2021-05-13", "Christi Himmelfahrt"),
            ("2021-05-24", "Pfingstmontag"),
            ("2021-06-03", "Fronleichnam"),
            ("2021-08-15", "Mariä Himmelfahrt"),
            ("2021-10-26", "Nationalfeiertag"),
            ("2021-11-01", "Allerheiligen"),
            ("2021-12-08", "Mariä Empfängnis"),
            ("2021-12-25", "Christtag"),
            ("2021-12-26", "Stephanstag"),
        )

    def test_bank_2021(self):
        self.assertBankHolidaysInYear(
            2021,
            ("2021-04-02", "Karfreitag"),
            ("2021-12-24", "Heiliger Abend"),
            ("2021-12-31", "Silvester"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Neujahr"),
            ("2022-01-06", "Heilige Drei Könige"),
            ("2022-03-19", "Hl. Josef"),
            ("2022-04-15", "Karfreitag"),
            ("2022-04-18", "Ostermontag"),
            ("2022-05-01", "Staatsfeiertag"),
            ("2022-05-04", "Hl. Florian"),
            ("2022-05-26", "Christi Himmelfahrt"),
            ("2022-06-06", "Pfingstmontag"),
            ("2022-06-16", "Fronleichnam"),
            ("2022-08-15", "Mariä Himmelfahrt"),
            ("2022-09-24", "Hl. Rupert"),
            ("2022-10-10", "Tag der Volksabstimmung"),
            ("2022-10-26", "Nationalfeiertag"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-11-11", "Hl. Martin"),
            ("2022-11-15", "Hl. Leopold"),
            ("2022-12-08", "Mariä Empfängnis"),
            ("2022-12-24", "Heiliger Abend"),
            ("2022-12-25", "Christtag"),
            ("2022-12-26", "Stephanstag"),
            ("2022-12-31", "Silvester"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-03-19", "Saint Joseph's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-04", "Saint Florian's Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-08-15", "Assumption Day"),
            ("2022-09-24", "Saint Rupert's Day"),
            ("2022-10-10", "1920 Carinthian plebiscite"),
            ("2022-10-26", "National Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-11", "Saint Martin's Day"),
            ("2022-11-15", "Saint Leopold's Day"),
            ("2022-12-08", "Immaculate Conception"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Saint Stephen's Day"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2022-03-19", "วันสมโภชนักบุญโยเซฟ"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-05-01", "วันแรงงาน"),
            ("2022-05-04", "วันสมโภชนักบุญฟลอเรียน"),
            ("2022-05-26", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2022-06-06", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2022-06-16", "วันสมโภชพระคริสตวรกาย"),
            ("2022-08-15", "วันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2022-09-24", "วันสมโภชนักบุญรูเพิร์ตแห่งซาลซ์บูร์ก"),
            ("2022-10-10", "วันครบรอบการลงประชามติคารินเทีย"),
            ("2022-10-26", "วันชาติออสเตรีย"),
            ("2022-11-01", "วันสมโภชนักบุญทั้งหลาย"),
            ("2022-11-11", "วันสมโภชนักบุญมาร์ติน"),
            ("2022-11-15", "วันสมโภชนักบุญลีโอโพลด์"),
            ("2022-12-08", "วันสมโภชแม่พระผู้ปฏิสนธินิรมล"),
            ("2022-12-24", "วันคริสต์มาสอีฟ"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "วันสมโภชนักบุญสเตเฟน"),
            ("2022-12-31", "วันสิ้นปี"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "Богоявлення"),
            ("2022-03-19", "День Святого Йосипа"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-04", "День Святого Флоріана"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-16", "Свято Тіла і Крові Христових"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-09-24", "День Святого Руперта"),
            ("2022-10-10", "Річниця референдуму 1920 року в Карінтії"),
            ("2022-10-26", "Національне свято"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-11", "День Святого Мартина"),
            ("2022-11-15", "День Святого Леопольда"),
            ("2022-12-08", "Непорочне зачаття Діви Марії"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "День Святого Стефана"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.belarus import Belarus
from tests.common import CommonCountryTests, WorkingDayTests


class TestBelarus(CommonCountryTests, WorkingDayTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Belarus)

    def test_public_2018(self):
        # http://calendar.by/procal.php?year=2018
        # https://www.officeholidays.com/countries/belarus/index.php
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "Новы год"),
            ("2018-01-02", "Выходны (перанесены з 20.01.2018)"),
            ("2018-01-07", "Нараджэнне Хрыстова (праваслаўнае Раство)"),
            ("2018-03-08", "Дзень жанчын"),
            ("2018-03-09", "Выходны (перанесены з 03.03.2018)"),
            ("2018-04-01", "Каталiцкi Вялiкдзень"),
            ("2018-04-08", "Праваслаўны Вялiкдзень"),
            ("2018-04-16", "Выходны (перанесены з 14.04.2018)"),
            ("2018-04-17", "Радаўніца"),
            ("2018-04-30", "Выходны (перанесены з 28.04.2018)"),
            ("2018-05-01", "Свята працы"),
            ("2018-05-09", "Дзень Перамогі"),
            ("2018-07-02", "Выходны (перанесены з 07.07.2018)"),
            ("2018-07-03", "Дзень Незалежнасці Рэспублікі Беларусь (Дзень Рэспублікі)"),
            ("2018-11-07", "Дзень Кастрычніцкай рэвалюцыі"),
            ("2018-12-24", "Выходны (перанесены з 22.12.2018)"),
            ("2018-12-25", "Нараджэнне Хрыстова (каталіцкае Раство)"),
            ("2018-12-31", "Выходны (перанесены з 29.12.2018)"),
        )

    def test_new_years_day(self):
        name = "Новы год"

        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertHolidayName(name, (f"{year}-01-02" for year in range(2020, self.end_year)))
        self.assertNoHolidayName(name, (f"{year}-01-02" for year in range(self.start_year, 2020)))

    def test_orthodox_christmas_day(self):
        self.assertHolidayName(
            "Нараджэнне Хрыстова (праваслаўнае Раство)",
            (f"{year}-01-07" for year in self.full_range),
        )

    def test_day_of_fatherland_defenders(self):
        name = "Дзень абаронцаў Айчыны і Узброеных Сіл Рэспублікі Беларусь"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-02-23" for year in self.full_range))

    def test_womens_day(self):
        self.assertHolidayName("Дзень жанчын", (f"{year}-03-08" for year in self.full_range))

    def test_constitution_day(self):
        name = "Дзень Канстытуцыі"

        self.assertWorkdayHolidayName(
            name, (f"{year}-03-15" for year in range(1999, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1999))
        self.assertHolidayName(name, (f"{year}-03-15" for year in range(1995, 1999)))
        self.assertNoHolidayName(name, range(self.start_year, 1995), range(1999, self.end_year))

    def test_labor_day(self):
        self.assertHolidayName("Свята працы", (f"{year}-05-01" for year in self.full_range))

    def test_day_of_unity_of_the_peoples_of_belarus_and_russia(self):
        name = "Дзень яднання народаў Беларусі і Расіі"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-04-02" for year in range(1996, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1996))

    def test_catholic_easter(self):
        name = "Каталiцкi Вялiкдзень"

        # https://calendar.by/content.php?id=19
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, range(1992, self.end_year))
        self.assertNoHolidayName(name, 1991)

    def test_orthodox_easter(self):
        name = "Праваслаўны Вялiкдзень"

        # https://calendar.by/content.php?id=19
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, range(1992, self.end_year))
        self.assertNoHolidayName(name, 1991)

    def test_national_symbol_day(self):
        name = "Дзень Дзяржаўнага сцяга, Дзяржаўнага герба і Дзяржаўнага гімна Рэспублікі Беларусь"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            "2020-05-10",
            "2021-05-09",
            "2022-05-08",
            "2023-05-14",
            "2024-05-12",
            "2025-05-11",
        )
        self.assertWorkdayHolidayName(name, range(1998, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1998))

    def test_day_of_the_republic(self):
        name = "Дзень Незалежнасці Рэспублікі Беларусь (Дзень Рэспублікі)"

        self.assertHolidayName(name, (f"{year}-07-27" for year in range(self.start_year, 1997)))
        self.assertHolidayName(name, (f"{year}-07-03" for year in range(1997, self.end_year)))
        self.assertNoHolidayName(name, (f"{year}-07-27" for year in range(1997, self.end_year)))
        self.assertNoHolidayName(name, (f"{year}-07-03" for year in range(self.start_year, 1997)))

    def test_day_of_peoples_unity(self):
        name = "Дзень народнага адзінства"

        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-09-17" for year in range(2021, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2021))

    def test_october_revolution_day(self):
        name = "Дзень Кастрычніцкай рэвалюцыі"

        self.assertHolidayName(name, (f"{year}-11-07" for year in range(1995, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1995))

    def test_radunitsa(self):
        name = "Радаўніца"

        # https://calendar.by/content.php?id=20
        self.assertHolidayName(
            name,
            "2020-04-28",
            "2021-05-11",
            "2022-05-03",
            "2023-04-25",
            "2024-05-14",
            "2025-04-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_dzyady(self):
        name = "Дзень памяці"

        self.assertWorkdayHolidayName(
            name, (f"{year}-11-02" for year in range(1998, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1998))
        self.assertHolidayName(name, (f"{year}-11-02" for year in range(1992, 1998)))
        self.assertNoHolidayName(name, 1991, range(1998, self.end_year))

    def test_catholic_christmas_day(self):
        self.assertHolidayName(
            "Нараджэнне Хрыстова (каталіцкае Раство)",
            (f"{year}-12-25" for year in self.full_range),
        )

    def test_substituted_holidays(self):
        self.assertHoliday(
            "1998-01-02",
            "1998-04-27",
            "1999-01-08",
            "1999-04-19",
            "2000-05-08",
            "2000-11-06",
            "2001-01-02",
            "2001-03-09",
            "2001-04-23",
            "2001-04-30",
            "2001-07-02",
            "2001-12-24",
            "2001-12-31",
            "2002-01-02",
            "2002-05-10",
            "2002-11-08",
            "2003-01-06",
            "2003-05-05",
            "2004-01-02",
            "2004-01-05",
            "2004-01-06",
            "2004-04-19",
            "2005-03-07",
            "2006-01-02",
            "2006-05-08",
            "2006-11-06",
            "2007-01-02",
            "2007-03-09",
            "2007-04-16",
            "2007-04-30",
            "2007-07-02",
            "2007-12-24",
            "2007-12-31",
            "2008-01-02",
            "2008-05-05",
            "2008-07-04",
            "2008-12-26",
            "2009-01-02",
            "2009-04-27",
            "2010-01-08",
            "2010-04-12",
            "2010-05-10",
            "2011-03-07",
            "2011-05-02",
            "2012-03-09",
            "2012-04-23",
            "2012-07-02",
            "2012-12-24",
            "2012-12-31",
            "2013-01-02",
            "2013-05-10",
            "2014-01-02",
            "2014-01-06",
            "2014-04-30",
            "2014-07-04",
            "2014-12-26",
            "2015-01-02",
            "2015-04-20",
            "2016-01-08",
            "2016-03-07",
            "2017-01-02",
            "2017-04-24",
            "2017-05-08",
            "2017-11-06",
            "2018-01-02",
            "2018-03-09",
            "2018-04-16",
            "2018-04-30",
            "2018-07-02",
            "2018-12-24",
            "2018-12-31",
            "2019-05-06",
            "2019-05-08",
            "2019-11-08",
            "2020-01-06",
            "2020-04-27",
            "2021-01-08",
            "2021-05-10",
            "2022-03-07",
            "2022-05-02",
            "2023-04-24",
            "2023-05-08",
            "2023-11-06",
            "2024-05-13",
            "2024-11-08",
            "2025-01-06",
            "2025-04-28",
            "2025-07-04",
            "2025-12-26",
            "2026-04-20",
        )

    def test_workdays(self):
        self.assertWorkingDay(
            "1998-01-10",
            "1998-04-25",
            "1999-01-16",
            "1999-04-17",
            "2000-05-13",
            "2000-11-11",
            "2001-01-20",
            "2001-03-03",
            "2001-04-21",
            "2001-04-28",
            "2001-07-07",
            "2001-12-22",
            "2001-12-29",
            "2002-01-05",
            "2002-05-18",
            "2002-11-16",
            "2003-01-04",
            "2003-05-03",
            "2004-01-10",
            "2004-01-17",
            "2004-01-31",
            "2004-04-17",
            "2005-03-12",
            "2006-01-21",
            "2006-05-06",
            "2006-11-04",
            "2006-12-30",
            "2007-03-17",
            "2007-04-14",
            "2007-05-05",
            "2007-07-07",
            "2007-12-22",
            "2007-12-29",
            "2008-01-12",
            "2008-05-03",
            "2008-06-28",
            "2008-12-20",
            "2009-01-10",
            "2009-04-25",
            "2010-01-23",
            "2010-04-17",
            "2010-05-15",
            "2011-03-12",
            "2011-05-14",
            "2012-03-11",
            "2012-04-28",
            "2012-06-30",
            "2012-12-22",
            "2012-12-29",
            "2013-01-05",
            "2013-05-18",
            "2014-01-04",
            "2014-01-11",
            "2014-05-03",
            "2014-07-12",
            "2014-12-20",
            "2015-01-10",
            "2015-04-25",
            "2016-01-16",
            "2016-03-05",
            "2017-01-21",
            "2017-04-29",
            "2017-05-06",
            "2017-11-04",
            "2018-01-20",
            "2018-03-03",
            "2018-04-14",
            "2018-04-28",
            "2018-07-07",
            "2018-12-22",
            "2018-12-29",
            "2019-05-04",
            "2019-05-11",
            "2019-11-16",
            "2020-01-04",
            "2020-04-04",
            "2021-01-16",
            "2021-05-15",
            "2022-03-12",
            "2022-05-14",
            "2023-04-29",
            "2023-05-13",
            "2023-11-11",
            "2024-05-18",
            "2024-11-16",
            "2025-01-11",
            "2025-04-26",
            "2025-07-12",
            "2025-12-20",
            "2026-04-25",
        )

        for year, dts in {
            2006: (
                "2006-01-21",
                "2006-05-06",
                "2006-11-04",
                "2006-12-30",
            ),
        }.items():
            self.assertWorkingDay(Belarus(years=year), dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Новы год"),
            ("2022-01-02", "Новы год"),
            ("2022-01-07", "Нараджэнне Хрыстова (праваслаўнае Раство)"),
            ("2022-02-23", "Дзень абаронцаў Айчыны і Узброеных Сіл Рэспублікі Беларусь"),
            ("2022-03-07", "Выходны (перанесены з 12.03.2022)"),
            ("2022-03-08", "Дзень жанчын"),
            ("2022-03-15", "Дзень Канстытуцыі"),
            ("2022-04-02", "Дзень яднання народаў Беларусі і Расіі"),
            ("2022-04-17", "Каталiцкi Вялiкдзень"),
            ("2022-04-24", "Праваслаўны Вялiкдзень"),
            ("2022-05-01", "Свята працы"),
            ("2022-05-02", "Выходны (перанесены з 14.05.2022)"),
            ("2022-05-03", "Радаўніца"),
            (
                "2022-05-08",
                "Дзень Дзяржаўнага сцяга, Дзяржаўнага герба і "
                "Дзяржаўнага гімна Рэспублікі Беларусь",
            ),
            ("2022-05-09", "Дзень Перамогі"),
            ("2022-07-03", "Дзень Незалежнасці Рэспублікі Беларусь (Дзень Рэспублікі)"),
            ("2022-09-17", "Дзень народнага адзінства"),
            ("2022-11-02", "Дзень памяці"),
            ("2022-11-07", "Дзень Кастрычніцкай рэвалюцыі"),
            ("2022-12-25", "Нараджэнне Хрыстова (каталіцкае Раство)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "New Year's Day"),
            ("2022-01-07", "Orthodox Christmas Day"),
            (
                "2022-02-23",
                "Day of the Fatherland's Defenders and "
                "the Armed Forces of the Republic of Belarus",
            ),
            ("2022-03-07", "Day off (substituted from 03/12/2022)"),
            ("2022-03-08", "Women's Day"),
            ("2022-03-15", "Constitution Day"),
            ("2022-04-02", "Day of Unity of the Peoples of Belarus and Russia"),
            ("2022-04-17", "Catholic Easter"),
            ("2022-04-24", "Orthodox Easter"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Day off (substituted from 05/14/2022)"),
            ("2022-05-03", "Radunitsa (Day of Rejoicing)"),
            (
                "2022-05-08",
                "Day of the National Coat of Arms of the Republic of Belarus, "
                "the National Flag of the Republic of Belarus and "
                "the National Anthem of the Republic of Belarus",
            ),
            ("2022-05-09", "Victory Day"),
            ("2022-07-03", "Independence Day of the Republic of Belarus (Day of the Republic)"),
            ("2022-09-17", "Day of People's Unity"),
            ("2022-11-02", "Dzyady (All Souls' Day)"),
            ("2022-11-07", "October Revolution Day"),
            ("2022-12-25", "Catholic Christmas Day"),
        )

    def test_l10n_ru(self):
        self.assertLocalizedHolidays(
            "ru",
            ("2022-01-01", "Новый год"),
            ("2022-01-02", "Новый год"),
            ("2022-01-07", "Рождество Христово (православное Рождество)"),
            ("2022-02-23", "День защитников Отечества и Вооруженных Сил Республики Беларусь"),
            ("2022-03-07", "Выходной (перенесено с 12.03.2022)"),
            ("2022-03-08", "День женщин"),
            ("2022-03-15", "День Конституции"),
            ("2022-04-02", "День единения народов Беларуси и России"),
            ("2022-04-17", "Католическая Пасха"),
            ("2022-04-24", "Православная Пасха"),
            ("2022-05-01", "Праздник труда"),
            ("2022-05-02", "Выходной (перенесено с 14.05.2022)"),
            ("2022-05-03", "Радуница"),
            (
                "2022-05-08",
                "День Государственного флага, Государственного герба и "
                "Государственного гимна Республики Беларусь",
            ),
            ("2022-05-09", "День Победы"),
            ("2022-07-03", "День Независимости Республики Беларусь (День Республики)"),
            ("2022-09-17", "День народного единства"),
            ("2022-11-02", "День памяти"),
            ("2022-11-07", "День Октябрьской революции"),
            ("2022-12-25", "Рождество Христово (католическое Рождество)"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-02", "วันขึ้นปีใหม่"),
            ("2022-01-07", "วันประสูติของพระคริสต์ (คริสต์มาสนิกายออร์โธดอกซ์)"),
            ("2022-02-23", "วันพิทักษ์ปิตุภูมิและกองทัพแห่งสาธารณรัฐเบลารุส"),
            ("2022-03-07", "วันหยุด (แทน 12/03/2022)"),
            ("2022-03-08", "วันสตรี"),
            ("2022-03-15", "วันรัฐธรรมนูญ"),
            ("2022-04-02", "วันแห่งความสามัคคีของประชาชนเบลารุสและรัสเซีย"),
            ("2022-04-17", "วันอีสเตอร์นิกายคาทอลิก"),
            ("2022-04-24", "วันอีสเตอร์นิกายออร์โธดอกซ์"),
            ("2022-05-01", "วันแรงงาน"),
            ("2022-05-02", "วันหยุด (แทน 14/05/2022)"),
            ("2022-05-03", "ราเดาว์นิตซา (วันแห่งความยินดี)"),
            ("2022-05-08", "วันธงชาติ ตราแผ่นดิน และเพลงชาติแห่งสาธารณรัฐเบลารุส"),
            ("2022-05-09", "วันแห่งชัยชนะ"),
            ("2022-07-03", "วันประกาศอิสรภาพแห่งสาธารณรัฐเบลารุส (วันสาธารณรัฐ)"),
            ("2022-09-17", "วันแห่งความสามัคคีของประชาชน"),
            ("2022-11-02", "ดียาดี (วันภาวนาอุทิศแด่ผู้ล่วงลับ)"),
            ("2022-11-07", "วันครบรอบการปฏิวัติเดือนตุลาคม"),
            ("2022-12-25", "วันประสูติของพระคริสต์ (คริสต์มาสนิกายคาทอลิก)"),
        )
